#ifndef WIRECELL_DRIFTER
#define WIRECELL_DRIFTER

#include "WireCellIface/IDrifter.h"
#include "WireCellIface/IConfigurable.h"
#include "WireCellUtil/Units.h"

#include <deque>

namespace WireCell {

    /** A Drifter takes inserted depositions, drifts and buffers as
     * needed and reproduces new depositions drifted to a new
     * location.
     */
    class Drifter : public IDrifter, public IConfigurable {
    public:
	/// Create a drifter that will drift charge to a given
	/// location at a given drift velocity.

	Drifter(double x_location = 0*units::meter,
		double drift_velocity = 1.6*units::mm/units::microsecond);
	// fixme: make configureable

	/// WireCell::IDrifter interface.
	virtual void reset();
	virtual bool operator()(const input_pointer& depo, output_queue& outq);

	/// WireCell::IConfigurable interface.
	virtual void configure(const WireCell::Configuration& config);
	virtual WireCell::Configuration default_configuration() const;

    private:

	double m_location, m_drift_velocity;
	bool m_eoi;		// end of input

	// Input buffer sorted by proper time
	DepoTauSortedSet m_input;

	double proper_time(IDepo::pointer depo);

	bool m_eos;
    };


}

#endif
